/***************************************************************************
 *                GOnOffBin.hpp - ON-OFF measurement class                 *
 * ----------------------------------------------------------------------- *
 *  copyright (C) 2009-2013 by Jurgen Knodlseder                           *
 * ----------------------------------------------------------------------- *
 *                                                                         *
 *  This program is free software: you can redistribute it and/or modify   *
 *  it under the terms of the GNU General Public License as published by   *
 *  the Free Software Foundation, either version 3 of the License, or      *
 *  (at your option) any later version.                                    *
 *                                                                         *
 *  This program is distributed in the hope that it will be useful,        *
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of         *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          *
 *  GNU General Public License for more details.                           *
 *                                                                         *
 *  You should have received a copy of the GNU General Public License      *
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.  *
 *                                                                         *
 ***************************************************************************/
/**
 * @file GOnOffBin.hpp
 * @brief ON-OFF measurement interface definition.
 * @author Pierrick Martin
 */

#ifndef GONOFFBIN_HPP
#define GONOFFBIN_HPP

/* __ Includes ___________________________________________________________ */
#include <string>
#include "GBase.hpp"

/* __ Forward declarations _______________________________________________ */
class GSkyRegion;
class GSkyRegions;
class GEnergy;
class GObservation;


/***********************************************************************//**
* @class GOnOffBin
*
* @brief ON-OFF measurement class.
*
* This class holds all low level information about an ON-OFF measurement,
* such as counts or acceptances for given ON and OFF regions. Higher level
* information such as fluxes is obtained from a subsequent fitting stage.
*
* This is a basic element in the implementation of classical data analysis
* methods for TeV data from existing Cherenkov telescopes such as H.E.S.S.,
* VERITAS, or MAGIC. Spectral or morphological analyses are performed on
* arrays of such a class.
*
* The class is built on the assumption that there is only one ON region but 
* multiple OFF regions that are thus stored in a sky region container.
*
* Note : The sky regions and energy members must be created for each 
* instance of GOnOffBin and not just be pointers to the objects passed
* on instantiation, because the latter may be erased/overwritten in a loop
* over observation runs when filling the GOnOffBin array.
***************************************************************************/
class GOnOffBin : public GBase {
	
public:
    // Constructors and destructors
    GOnOffBin(void);
	explicit GOnOffBin(const GObservation& obs, const GSkyRegion& on, const GSkyRegions& off, const GEnergy emin, const GEnergy emax);
    GOnOffBin(const GOnOffBin& bin);
	~GOnOffBin(void);
	
    // Operators
    virtual GOnOffBin& operator= (const GOnOffBin& bin);
	
    // Basic methods
	void            clear(void);
    GOnOffBin*      clone(void) const;
    std::string     type(void) const;
    std::string     print(const GChatter& chatter = NORMAL) const;
	
	// Specific methods
	GEnergy*        emin(void) const { return m_emin; }
	GEnergy*        emax(void) const { return m_emax; }
	GSkyRegion*     onregion(void) const { return m_onreg; } 
	GSkyRegions*    offregion(void) const { return m_offreg; }
	const double    alpha(void) const { return m_alpha; }
	const double    onacc(void) const { return m_onacc; }
	const double    offacc(void) const { return m_offacc; }
	const double    ontime(void) const { return m_ontime; }
	const double    offtime(void) const { return m_offtime; }
	const int       oncts(void) const { return m_oncts; }
	const int       offcts(void) const { return m_offcts; }
	
protected:
    // Protected methods
    void init_members(void);
    void copy_members(const GOnOffBin& bin);
    void free_members(void);
	
	// Protected members
	std::string    m_obsid;       //!< Observation identifier
	int            m_oncts;       //!< Number of counts in ON region
	int            m_offcts;      //!< Number of counts in OFF region
	double         m_onacc;       //!< Acceptance in ON region
	double         m_offacc;      //!< Acceptance in OFF region
	double         m_ontime;      //!< Exposure time in ON region
	double         m_offtime;     //!< Exposure time in OFF region
	double         m_alpha;       //!< Ratio of ON/OFF acceptances
	std::string    m_method;      //!< Method used for background estimation
	GSkyRegion*    m_onreg        //!< ON sky region object
	GSkyRegions*   m_offreg       //!< OFF sky regions container
	GEnergy*       m_emin         //!< Lower energy bound
	GEnergy*       m_emax         //!< Upper energy bound
};

#endif /* GONOFFBIN_HPP */
